﻿Option Strict On
Imports Microsoft.Kinect
Imports Coding4Fun.Kinect.Wpf
Imports System.Windows.Threading
Imports System.IO

Class MainWindow
    Dim kinect As KinectSensor
    Dim flag As Boolean = False

    Dim outTime As String = String.Empty
    Dim inTime As String
    Dim myTimer As DispatcherTimer
    Dim imageFileName As String

    Private Sub MainWindow_Loaded(sender As Object, e As System.Windows.RoutedEventArgs) Handles Me.Loaded
        Dim dir As String = Environment.GetFolderPath(Environment.SpecialFolder.MyPictures)

        If Directory.Exists(Path.Combine(dir, "KINECT")) = False Then
            Directory.CreateDirectory(Path.Combine(dir, "KINECT"))
        End If
        Dim fileDir = Path.Combine(dir, "KINECT")

        If File.Exists(Path.Combine(fileDir, "ImageFileList.xml")) = True Then
            Button1.IsEnabled = True
        Else
            Button1.IsEnabled = False
        End If



        myTimer = New DispatcherTimer
        myTimer.Interval = New TimeSpan(1000)

        AddHandler myTimer.Tick, Sub(timerSender As Object, timerArgs As EventArgs)
                                     inTextBlock.Text = DateTime.Now.ToString("yyyy年MM月dd日HH時mm分ss秒")
                                 End Sub
        myTimer.Start()


        If KinectSensor.KinectSensors.Count = 0 Then
            MessageBox.Show("KINECTが接続されておりません。")
            Exit Sub
        Else
            kinect = KinectSensor.KinectSensors(0)
            kinect.ColorStream.Enable(ColorImageFormat.RgbResolution640x480Fps30)
            kinect.SkeletonStream.Enable()


            AddHandler kinect.AllFramesReady, AddressOf kinect_AllFramesReady
            kinect.Start()
        End If
    End Sub

    Private Sub kinect_AllFramesReady(sender As Object, e As AllFramesReadyEventArgs)
        Using myImage As ColorImageFrame = e.OpenColorImageFrame
            Image1.Source = myImage.ToBitmapSource
        End Using

        TextBlock2.Text = flag.ToString
        If flag = True Then
            myTimer.Stop()
            ListBox1.Items.Clear()
            ListBox1.Items.Add(DateTime.Now.ToString("yyyy年MM月dd日HH時mm分ss秒"))
            ListBox1.SelectedIndex = 0
            DataSave()
        Else
            TextBlock1.Text = String.Empty
            myTimer.Start()
        End If
        mySkeletonShow(e)
    End Sub

    Private Sub mySkeletonShow(e As AllFramesReadyEventArgs)

        Dim mySkeletonFrame As SkeletonFrame = e.OpenSkeletonFrame
        If mySkeletonFrame Is Nothing = False Then
            Dim skeletonData As Skeleton() = New Skeleton(mySkeletonFrame.SkeletonArrayLength - 1) {}
            mySkeletonFrame.CopySkeletonDataTo(skeletonData)

            For Each mySkeleton In skeletonData
                If mySkeleton.TrackingState = SkeletonTrackingState.Tracked Then
                    flag = True
                    myTimer.Stop()
                End If
            Next
        Else
            flag = False
            myTimer.Start()
        End If
    End Sub

    Private Sub DataSave()
        If ListBox1.Items.Count > 0 Then

            Dim bmp = New RenderTargetBitmap(CInt(Image1.Width), CInt(Image1.Height), 96, 96, PixelFormats.Pbgra32)
            bmp.Render(Image1)
            Dim myJpg = New JpegBitmapEncoder
            myJpg.Frames.Add(BitmapFrame.Create(bmp))

            Dim dir As String = Environment.GetFolderPath(Environment.SpecialFolder.MyPictures)
            Dim saveDir As String = Path.Combine(dir, "KINECT")

            Using fs As FileStream = File.Open(Path.Combine(saveDir, imageFileName), FileMode.Create)
                myJpg.Save(fs)
            End Using


            outTime = ListBox1.SelectedItem.ToString
            inTime = inTextBlock.Text

            If File.Exists(Path.Combine(saveDir, "ImageFileList.xml")) = False Then
                If inTime <> ListBox1.SelectedItem.ToString Then
                    Dim xmldoc As XDocument = <?xml version="1.0" encoding="utf-8"?>
                                              <人物記録>
                                                  <情報>
                                                      <入時間><%= inTime %></入時間>
                                                      <画像名><%= imageFileName %></画像名>
                                                  </情報>
                                              </人物記録>
                    xmldoc.Save(Path.Combine(saveDir, "ImageFileList.xml"))
                    Button1.IsEnabled = True
                    TextBlock1.Text = "記録中。"
                End If
            Else
                If inTime <> ListBox1.SelectedItem.ToString Then
                    Dim readXmldoc As XElement = XElement.Load(Path.Combine(saveDir, "ImageFileList.xml"))
                    Dim addXml As XElement =
                        <情報>
                            <入時間><%= inTime %></入時間>
                            <画像名><%= imageFileName %></画像名>
                        </情報>
                    readXmldoc.Add(addXml)
                    readXmldoc.Save(Path.Combine(saveDir, "ImageFileList.xml"))
                    Button1.IsEnabled = True
                    TextBlock1.Text = "記録中。"
                End If
            End If

            'ListBox1.Items.Clear()
        End If
    End Sub

    Private Sub Button1_Click(sender As System.Object, e As System.Windows.RoutedEventArgs) Handles Button1.Click
        Dim dataPage As New DataShowWindow
        dataPage.Show()
    End Sub

    Private Sub ListBox1_SelectionChanged(sender As Object, e As System.Windows.Controls.SelectionChangedEventArgs) Handles ListBox1.SelectionChanged
        If ListBox1.Items.Count > 0 Then
            imageFileName = ListBox1.SelectedItem.ToString & ".jpg"
        End If
    End Sub

    Private Sub MainWindow_Closing(sender As Object, e As System.ComponentModel.CancelEventArgs) Handles Me.Closing
        If kinect Is Nothing = False Then
            If kinect.IsRunning = True Then
                RemoveHandler kinect.AllFramesReady, AddressOf kinect_AllFramesReady
                kinect.Stop()
                kinect.Dispose()
            End If
        End If
    End Sub
End Class
